#pragma once
#include <string>
#include <curl/curl.h>
#include"src/json.hpp"

#include <iostream>
#include <Windows.h>
#include <rpc.h>
#pragma comment(lib, "rpcrt4.lib")

#include <sstream>
#include <iomanip>

class MixpanelScript
{
public :
	void PostData(std::string EventName, std::string DistinctId, std::string InsertId,int error);
	std::string GetGUID();
	std::string GetCPUID();
private:
	
	std::string GuidToString(const GUID& guid);
};

